@@ -1,11 +1,14 @@
 import asyncio
 import functools
+import logging
 
 import aioredis
 from aioredis.pubsub import Receiver
 
 from aiopubsub.base import BasePubsub
 
+LOG = logging.getLogger(__name__)
+
 
 def init_conn(func):
     @functools.wraps(func)
@@ -29,7 +32,7 @@ class RedisBackend:
         super().__init__(**kwargs)
         self.host = host
         self.port = int(port)
-        self.db=db
+        self.db = db
         self._loop = loop
         create_connection_timeout = (float(socket_connect_timeout) if socket_connect_timeout else None)
         self.kwargs = {"db": int(db), "password": password,
@@ -41,6 +44,7 @@ class RedisBackend:
 
         self.__pool_lock = None
         self._pool: aioredis.ConnectionsPool = None
+        self._mpsc: Receiver = None
 
     @property
     def _pool_lock(self):
@@ -55,8 +59,16 @@ class RedisBackend:
         return conn
 
     async def _release_sub(self, _conn: aioredis.Redis):
+        if self._mpsc:
+            channels = [c.decode() for c in self._mpsc.channels.keys()]
+            if channels:
+                await _conn.unsubscribe(*channels)
+            patterns = [p.decode() for p in self._mpsc.patterns.keys()]
+            if patterns:
+                await _conn.punsubscribe(*patterns)
+            self._mpsc.stop()
+            self._mpsc = None
         self._pool.release(_conn.connection)
-        return
 
     async def _acquire_pub(self):
         await self._get_pool()
@@ -66,45 +78,53 @@ class RedisBackend:
 
     async def _release_pub(self, _conn: aioredis.Redis):
         self._pool.release(_conn.connection)
-        return
 
     @init_conn
     async def _unsubscribe(self, channel, *channels, _conn: aioredis.Redis = None):
         return await _conn.unsubscribe(channel, *channels)
 
+    @property
+    def mpsc(self):
+        if not self._mpsc:
+            self._mpsc = Receiver()
+        return self._mpsc
+
     @init_conn
     async def _subscribe(self, channel, *channels, _conn: aioredis.Redis = None):
-        mpsc = Receiver()
-        ns_channels = [mpsc.channel(c) for c in (channel, *channels)]
-        await _conn.subscribe(*ns_channels)
-        return mpsc
+        sub_channels = [self.mpsc.channel(c) for c in (channel, *channels)]
+        ret = await _conn.subscribe(*sub_channels)
+        LOG.info(f"sub: {sub_channels}, ret: {ret}")
+        return
 
     @init_conn
     async def _psubscribe(self, pattern, *patterns, _conn: aioredis.Redis = None):
-        mpsc = Receiver()
-        ns_patterns = [mpsc.pattern(p) for p in (pattern, *patterns)]
-        await _conn.psubscribe(*ns_patterns)
-        return mpsc
+        psub_patterns = [self.mpsc.pattern(p) for p in (pattern, *patterns)]
+        ret = await _conn.psubscribe(*psub_patterns)
+        LOG.info(f"psub: {psub_patterns}, ret: {ret}")
 
     @init_conn
     async def _punsubscribe(self, pattern, *patterns, _conn: aioredis.Redis = None):
         return await _conn.punsubscribe(pattern, *patterns)
 
-    async def _listen(self, _conn: Receiver = None):
-        """Listen for messages on channels this client has been subscribed to"""
-        async for channel, msg in _conn.iter():
-            if channel.is_pattern:
-                channel_name, data = msg
-                channel_name = channel_name.decode()
-                data = data.decode()
-                pattern = channel.name.decode()
-                type_ = "pmessage"
-            else:
-                pattern = None
-                type_ = "message"
-                data = msg.decode()
-                channel_name = channel.name.decode()
-            yield {"type": type_, "pattern": pattern, "channel": channel_name, "data": data}
+    @property
+    def subscribed(self):
+        return bool(self.mpsc and (self.mpsc.channels or self.mpsc.patterns))
+
+    async def _listen(self, _conn: aioredis.Redis = None):
+        while self.subscribed:
+            async for channel, msg in self.mpsc.iter():
+                if channel.is_pattern:
+                    channel_name, data = msg
+                    channel_name = channel_name.decode()
+                    data = data.decode()
+                    pattern = channel.name.decode()
+                    type_ = "pmessage"
+                else:
+                    pattern = None
+                    type_ = "message"
+                    data = msg.decode()
+                    channel_name = channel.name.decode()
+                yield {"type": type_, "pattern": pattern, "channel": channel_name, "data": data}
 
     @init_conn
     async def _publish(self, channel, message, _conn: aioredis.Redis = None):
